@@ -954,6 +954,12 @@ public:
 	}
 };
 
+// forward declarations
+namespace Burp
+{
+	class BurpTaskItem;
+};
+
 class BurpGlobals : public Firebird::ThreadData, public GblPool
 {
 public:
@@ -1205,7 +1211,7 @@ public:
 	Firebird::Array<UCHAR> gbl_dpb_data;
 	Firebird::UtilSvc* uSvc;
 	bool master;			// set for master thread only
-	void* taskItem;			// current task item, if any
+	Burp::BurpTaskItem* taskItem;			// current task item, if any.
 	ULONG verboseInterval;	// How many records should be backed up or restored before we show this message
 	bool flag_on_line;		// indicates whether we will bring the database on-line
 	bool firstMap;			// this is the first time we entered get_mapping()
