@@ -193,6 +193,7 @@ enum ConfigKey
 	KEY_MAX_PARALLEL_WORKERS,
 	KEY_OPTIMIZE_FOR_FIRST_ROWS,
 	KEY_OUTER_JOIN_CONVERSION,
+	KEY_SUBQUERY_CONVERSION,
 	MAX_CONFIG_KEY		// keep it last
 };
 
@@ -312,7 +313,8 @@ constexpr ConfigEntry entries[MAX_CONFIG_KEY] =
 	{TYPE_INTEGER,	"ParallelWorkers",			true,	1},
 	{TYPE_INTEGER,	"MaxParallelWorkers",		true,	1},
 	{TYPE_BOOLEAN,	"OptimizeForFirstRows",		false,	false},
-	{TYPE_BOOLEAN,	"OuterJoinConversion",		false,	true}
+	{TYPE_BOOLEAN,	"OuterJoinConversion",		false,	true},
+	{TYPE_BOOLEAN,	"SubQueryConversion",		false,	false}
 };
 
 
@@ -324,9 +326,15 @@ private:
 	static ConfigValue specialProcessing(ConfigKey key, ConfigValue val);
 
 	void loadValues(const ConfigFile& file, const char* srcName);
-	void setupDefaultConfig();
 	void checkValues();
 
+	// set default ServerMode and default values that didn't depends on ServerMode
+	void setupDefaultConfig();
+
+	// set default values that depends on ServerMode and actual values that was
+	// not set in config file
+	void fixDefaults();
+
 	// helper check-value functions
 	void checkIntForLoBound(ConfigKey key, SINT64 loBound, bool setDefault);
 	void checkIntForHiBound(ConfigKey key, SINT64 hiBound, bool setDefault);
@@ -642,6 +650,8 @@ public:
 	CONFIG_GET_PER_DB_BOOL(getOptimizeForFirstRows, KEY_OPTIMIZE_FOR_FIRST_ROWS);
 
 	CONFIG_GET_PER_DB_BOOL(getOuterJoinConversion, KEY_OUTER_JOIN_CONVERSION);
+
+	CONFIG_GET_PER_DB_BOOL(getSubQueryConversion, KEY_SUBQUERY_CONVERSION);
 };
 
 // Implementation of interface to access master configuration file
