@@ -47,6 +47,7 @@
 #include "../jrd/replication/Applier.h"
 #include "../jrd/replication/Manager.h"
 
+#include "../dsql/DsqlBatch.h"
 #include "../dsql/DsqlStatementCache.h"
 
 #include "../common/classes/fb_string.h"
@@ -287,9 +288,6 @@ Jrd::Attachment::~Attachment()
 
 	delete att_trace_manager;
 
-	for (unsigned n = 0; n < att_batches.getCount(); ++n)
-		att_batches[n]->resetHandle();
-
 	for (Function** iter = att_functions.begin(); iter < att_functions.end(); ++iter)
 	{
 		Function* const function = *iter;
@@ -448,6 +446,12 @@ void Jrd::Attachment::storeBinaryBlob(thread_db* tdbb, jrd_tra* transaction,
 	blob->BLB_close(tdbb);
 }
 
+void Jrd::Attachment::releaseBatches()
+{
+	while (att_batches.hasData())
+		delete att_batches.pop();
+}
+
 void Jrd::Attachment::releaseGTTs(thread_db* tdbb)
 {
 	if (!att_relations)
@@ -602,6 +606,12 @@ void Jrd::Attachment::resetSession(thread_db* tdbb, jrd_tra** traHandle)
 	}
 	catch (const Exception& ex)
 	{
+		if (att_ext_call_depth && !shutAtt)
+		{
+			flags.release(ATT_resetting);		// reset is incomplete - keep state
+			shutAtt = true;
+		}
+
 		if (shutAtt)
 			signalShutdown(isc_ses_reset_failed);
 
@@ -1135,6 +1145,18 @@ void Attachment::checkReplSetLock(thread_db* tdbb)
 
 void Attachment::invalidateReplSet(thread_db* tdbb, bool broadcast)
 {
+	if (broadcast)
+	{
+		// Signal other attachments about the changed state
+		if (att_repl_lock->lck_logical == LCK_none)
+			LCK_lock(tdbb, att_repl_lock, LCK_EX, LCK_WAIT);
+		else
+			LCK_convert(tdbb, att_repl_lock, LCK_EX, LCK_WAIT);
+	}
+
+	if (att_flags & ATT_repl_reset)
+		return;
+
 	att_flags |= ATT_repl_reset;
 
 	if (att_relations)
@@ -1146,15 +1168,6 @@ void Attachment::invalidateReplSet(thread_db* tdbb, bool broadcast)
 		}
 	}
 
-	if (broadcast)
-	{
-		// Signal other attachments about the changed state
-		if (att_repl_lock->lck_logical == LCK_none)
-			LCK_lock(tdbb, att_repl_lock, LCK_EX, LCK_WAIT);
-		else
-			LCK_convert(tdbb, att_repl_lock, LCK_EX, LCK_WAIT);
-	}
-
 	LCK_release(tdbb, att_repl_lock);
 }
 
@@ -1198,7 +1211,16 @@ bool Attachment::isProfilerActive()
 	return att_profiler_manager && att_profiler_manager->isActive();
 }
 
-void Attachment::releaseProfilerManager()
+void Attachment::releaseProfilerManager(thread_db* tdbb)
 {
-	att_profiler_manager.reset();
+	if (!att_profiler_manager)
+		return;
+
+	if (att_profiler_manager->haveListener())
+	{
+		EngineCheckout cout(tdbb, FB_FUNCTION);
+		att_profiler_manager.reset();
+	}
+	else
+		att_profiler_manager.reset();
 }
