@@ -609,7 +609,7 @@ public:
 		// We must zero-initialize this one
 		csb_repeat();
 
-		void activate(bool subStream = false);
+		void activate();
 		void deactivate();
 
 		Nullable<USHORT> csb_cursor_number;	// Cursor number for this stream
@@ -659,12 +659,9 @@ inline CompilerScratch::csb_repeat::csb_repeat()
 {
 }
 
-inline void CompilerScratch::csb_repeat::activate(bool subStream)
+inline void CompilerScratch::csb_repeat::activate()
 {
 	csb_flags |= csb_active;
-
-	if (subStream)
-		csb_flags |= csb_sub_stream;
 }
 
 inline void CompilerScratch::csb_repeat::deactivate()
