@@ -136,14 +136,10 @@ void InternalConnection::attach(thread_db* tdbb)
 	// Don't wrap raised errors. This is needed for backward compatibility.
 	setWrapErrors(false);
 
-	if (m_dpb.isEmpty())
-	{
-		m_isCurrent = true;
+	if (isCurrent())
 		m_attachment = attachment->getInterface();
-	}
 	else
 	{
-		m_isCurrent = false;
 		m_dbName = dbb->dbb_database_name.c_str();
 
 		// Avoid change of m_dpb by validatePassword() below
@@ -156,6 +152,7 @@ void InternalConnection::attach(thread_db* tdbb)
 			EngineCallbackGuard guard(tdbb, *this, FB_FUNCTION);
 			m_provider.reset(attachment->getProvider());
 			m_provider->addRef();
+			m_provider->setDbCryptCallback(&status, &m_cryptCallbackRedir);
 			m_attachment.assignRefNoIncr(m_provider->attachDatabase(&status, m_dbName.c_str(),
 				newDpb.getBufferLength(), newDpb.getBuffer()));
 		}
@@ -181,7 +178,7 @@ void InternalConnection::doDetach(thread_db* tdbb)
 	if (!m_attachment->getHandle())
 		return;
 
-	if (m_isCurrent)
+	if (isCurrent())
 	{
 		m_attachment = NULL;
 	}
@@ -220,7 +217,7 @@ bool InternalConnection::cancelExecution(bool /*forced*/)
 	if (!m_attachment->getHandle())
 		return false;
 
-	if (m_isCurrent)
+	if (isCurrent())
 		return true;
 
 	FbLocalStatus status;
@@ -231,9 +228,9 @@ bool InternalConnection::cancelExecution(bool /*forced*/)
 
 bool InternalConnection::resetSession(thread_db* tdbb)
 {
-	fb_assert(!m_isCurrent);
+	fb_assert(isCurrent());
 
-	if (m_isCurrent)
+	if (isCurrent())
 		return true;
 
 	FbLocalStatus status;
@@ -251,13 +248,13 @@ bool InternalConnection::resetSession(thread_db* tdbb)
 // b) is not current connection
 bool InternalConnection::isAvailable(thread_db* tdbb, TraScope /*traScope*/) const
 {
-	return !m_isCurrent ||
-		(m_isCurrent && (tdbb->getAttachment() == m_attachment->getHandle()));
+	return (!isCurrent()) ||
+		(isCurrent() && (tdbb->getAttachment() == m_attachment->getHandle()));
 }
 
 bool InternalConnection::validate(thread_db* tdbb)
 {
-	if (m_isCurrent)
+	if (isCurrent())
 		return true;
 
 	if (!m_attachment)
@@ -269,9 +266,9 @@ bool InternalConnection::validate(thread_db* tdbb)
 	return status.isSuccess();
 }
 
-bool InternalConnection::isSameDatabase(const PathName& dbName, ClumpletReader& dpb) const
+bool InternalConnection::isSameDatabase(const PathName& dbName, ClumpletReader& dpb, const CryptHash& ch) const
 {
-	if (m_isCurrent)
+	if (isCurrent())
 	{
 		const Attachment* att = m_attachment->getHandle();
 		const MetaString& attUser = att->getUserName();
@@ -294,7 +291,7 @@ bool InternalConnection::isSameDatabase(const PathName& dbName, ClumpletReader&
 		}
 	}
 
-	return Connection::isSameDatabase(dbName, dpb);
+	return Connection::isSameDatabase(dbName, dpb, ch);
 }
 
 Transaction* InternalConnection::doCreateTransaction()
