@@ -40,6 +40,7 @@
 #include "../dsql/ExprNodes.h"
 #include "../jrd/RecordSourceNodes.h"
 #include "../jrd/exe.h"
+#include "../jrd/recsrc/RecordSource.h"
 
 namespace Jrd {
 
@@ -118,10 +119,10 @@ public:
 		}
 	}
 
-	void activate(bool subStream = false)
+	void activate()
 	{
 		for (const auto stream : m_streams)
-			m_csb->csb_rpt[stream].activate(subStream);
+			m_csb->csb_rpt[stream].activate();
 	}
 
 	void deactivate()
@@ -255,9 +256,17 @@ public:
 
 	class ConjunctIterator
 	{
-		friend class Optimizer;
-
 	public:
+		ConjunctIterator(Conjunct* _begin, const Conjunct* _end)
+			: begin(_begin), end(_end)
+		{
+			rewind();
+		}
+
+		ConjunctIterator(const ConjunctIterator& other)
+			: begin(other.begin), end(other.end), iter(other.iter)
+		{}
+
 		operator BoolExprNode*() const
 		{
 			return iter->node;
@@ -313,16 +322,6 @@ public:
 		Conjunct* const begin;
 		const Conjunct* const end;
 		Conjunct* iter;
-
-		ConjunctIterator(Conjunct* _begin, const Conjunct* _end)
-			: begin(_begin), end(_end)
-		{
-			rewind();
-		}
-
-		explicit ConjunctIterator(const ConjunctIterator& other)
-			: begin(other.begin), end(other.end), iter(other.iter)
-		{}
 	};
 
 	ConjunctIterator getBaseConjuncts()
@@ -494,9 +493,16 @@ public:
 		return firstRows;
 	}
 
+	bool isSemiJoined() const
+	{
+		return (rse->flags & RseNode::FLAG_SEMI_JOINED) != 0;
+	}
+
+	RecordSource* applyBoolean(RecordSource* rsb, ConjunctIterator& iter);
 	RecordSource* applyLocalBoolean(RecordSource* rsb,
 									const StreamList& streams,
 									ConjunctIterator& iter);
+
 	bool checkEquiJoin(BoolExprNode* boolean);
 	bool getEquiJoinKeys(BoolExprNode* boolean,
 						 NestConst<ValueExprNode>* node1,
@@ -521,7 +527,7 @@ private:
 					RiverList& rivers,
 					SortNode** sortClause,
 					const PlanNode* planClause);
-	bool generateEquiJoin(RiverList& org_rivers);
+	bool generateEquiJoin(RiverList& rivers, JoinType joinType = INNER_JOIN);
 	void generateInnerJoin(StreamList& streams,
 						   RiverList& rivers,
 						   SortNode** sortClause,
@@ -548,7 +554,7 @@ private:
 	unsigned baseParentConjuncts = 0;		// number of conjuncts in our rse + distributed with parent, next are parent
 	unsigned baseMissingConjuncts = 0;		// number of conjuncts in our and parent rse, but without missing
 
-	StreamList compileStreams, bedStreams, keyStreams, subStreams, outerStreams;
+	StreamList compileStreams, bedStreams, keyStreams, outerStreams;
 	ConjunctList conjuncts;
 };
 
@@ -595,6 +601,7 @@ struct IndexScratchSegment
 	bool excludeUpper = false;					// exclude upper bound value from scan
 	unsigned scope = 0;							// highest scope level
 	segmentScanType scanType = segmentScanNone;	// scan type
+	SSHORT scale = 0;							// scale for SINT64/Int128-based segment of index
 
 	MatchedBooleanList matches;					// matched booleans
 };
@@ -612,8 +619,8 @@ struct IndexScratch
 	unsigned lowerCount = 0;
 	unsigned upperCount = 0;
 	unsigned nonFullMatchedSegments = 0;
-	bool usePartialKey = false;				// Use INTL_KEY_PARTIAL
-	bool useMultiStartingKeys = false;		// Use INTL_KEY_MULTI_STARTING
+	bool usePartialKey = false;					// Use INTL_KEY_PARTIAL
+	bool useMultiStartingKeys = false;			// Use INTL_KEY_MULTI_STARTING
 	bool useRootListScan = false;
 
 	Firebird::ObjectsArray<IndexScratchSegment> segments;
@@ -676,7 +683,7 @@ public:
 protected:
 	void analyzeNavigation(const InversionCandidateList& inversions);
 	bool betterInversion(const InversionCandidate* inv1, const InversionCandidate* inv2,
-						 bool ignoreUnmatched) const;
+						 bool navigation) const;
 	bool checkIndexCondition(index_desc& idx, MatchedBooleanList& matches) const;
 	bool checkIndexExpression(const index_desc* idx, ValueExprNode* node) const;
 	InversionNode* composeInversion(InversionNode* node1, InversionNode* node2,
@@ -777,7 +784,8 @@ class InnerJoin : private Firebird::PermanentStorage
 	{
 	public:
 		StreamInfo(MemoryPool& p, StreamType num)
-			: number(num), indexedRelationships(p)
+			: number(num), baseMatches(p), baseDependentFromStreams(p),
+			  indexedRelationships(p)
 		{}
 
 		bool isIndependent() const
@@ -824,6 +832,9 @@ class InnerJoin : private Firebird::PermanentStorage
 		bool used = false;
 		unsigned previousExpectedStreams = 0;
 
+		MatchedBooleanList baseMatches;
+		SortedStreamList baseDependentFromStreams;
+
 		IndexedRelationships indexedRelationships;
 	};
 
@@ -831,7 +842,7 @@ class InnerJoin : private Firebird::PermanentStorage
 
 	struct JoinedStreamInfo
 	{
-		static const unsigned MAX_EQUI_MATCHES = 4;
+		static const unsigned MAX_EQUI_MATCHES = 8;
 
 		void reset (StreamType num)
 		{
