@@ -55,29 +55,6 @@ using namespace Jrd;
 
 namespace
 {
-	ValueExprNode* injectCast(CompilerScratch* csb,
-							  ValueExprNode* value, CastNode*& cast,
-							  const dsc& desc)
-	{
-		// If the indexed column is of type int64, then we need to inject
-		// an extra cast to deliver the scale value to the BTR level
-
-		if (value && desc.dsc_dtype == dtype_int64)
-		{
-			if (!cast)
-			{
-				cast = FB_NEW_POOL(csb->csb_pool) CastNode(csb->csb_pool);
-				cast->source = value;
-				cast->castDesc = desc;
-				cast->impureOffset = csb->allocImpure<impure_value>();
-			}
-
-			value = cast;
-		}
-
-		return value;
-	}
-
 	ValueExprNode* invertBoolValue(CompilerScratch* csb, ValueExprNode* value)
 	{
 		// Having a condition (<field> != <boolean value>),
@@ -629,6 +606,7 @@ void Retrieval::analyzeNavigation(const InversionCandidateList& inversions)
 			candidate->cost = DEFAULT_INDEX_COST + indexScratch.cardinality;
 			candidate->indexes = 1;
 			candidate->scratch = &indexScratch;
+			candidate->nonFullMatchedSegments = indexScratch.segments.getCount();
 			tempCandidates.add(candidate);
 		}
 
@@ -651,7 +629,7 @@ void Retrieval::analyzeNavigation(const InversionCandidateList& inversions)
 
 bool Retrieval::betterInversion(const InversionCandidate* inv1,
 								const InversionCandidate* inv2,
-								bool ignoreUnmatched) const
+								bool navigation) const
 {
 	// Return true if inversion1 is *better* than inversion2.
 	// It's mostly about the retrieval cost, but other aspects are also taken into account.
@@ -701,19 +679,33 @@ bool Retrieval::betterInversion(const InversionCandidate* inv1,
 			{
 				// If the "same" costs then compare with the nr of unmatched segments,
 				// how many indexes and matched segments. First compare number of indexes.
+
 				int diff = (inv1->indexes - inv2->indexes);
 
 				if (diff == 0)
 				{
 					// For the same number of indexes compare number of matched segments.
 					// Note the inverted condition: the more matched segments the better.
+
 					diff = (inv2->matchedSegments - inv1->matchedSegments);
 
-					if (diff == 0 && !ignoreUnmatched)
+					if (diff == 0 && !navigation)
 					{
-						// For the same number of matched segments
-						// compare ones that aren't full matched
+						// For the same number of matched segments compare ones that aren't full matched.
+						//
+						// However, unmatched segments and small cost difference do not matter
+						// if we already know the first retrieval being usable for navigation.
+
 						diff = (inv1->nonFullMatchedSegments - inv2->nonFullMatchedSegments);
+
+						if (diff == 0)
+						{
+							// For inversions with nearly the same cost but without other preferences found,
+							// return the actually cheaper inversion (based on cost only)
+
+							if (cost1 < cost2)
+								return true;
+						}
 					}
 				}
 
@@ -756,6 +748,9 @@ bool Retrieval::checkIndexCondition(index_desc& idx, MatchedBooleanList& matches
 
 		for (iter.rewind(); iter.hasData(); ++iter)
 		{
+			if (!iter->containsStream(stream))
+				continue;
+
 			if (matchSubset(boolean, *iter))
 			{
 				matches.add(*iter);
@@ -773,6 +768,9 @@ bool Retrieval::checkIndexCondition(index_desc& idx, MatchedBooleanList& matches
 		{
 			for (iter.rewind(); iter.hasData(); ++iter)
 			{
+				if (!iter->containsStream(stream))
+					continue;
+
 				const auto cmpNode = nodeAs<ComparativeBoolNode>(*iter);
 				if (cmpNode && cmpNode->blrOp != blr_equiv)
 				{
@@ -829,6 +827,8 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 									   unsigned scope) const
 {
 	const double cardinality = csb->csb_rpt[stream].csb_cardinality;
+	fb_assert(cardinality);
+	const double minSelectivity = MIN(MAXIMUM_SELECTIVITY / cardinality, DEFAULT_SELECTIVITY);
 
 	// Walk through indexes to calculate selectivity / candidate
 	MatchedBooleanList matches;
@@ -858,6 +858,8 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 			{
 				const auto& segment = scratch.segments[j];
 
+				auto scanType = segment.scanType;
+
 				if (segment.scope == scope)
 					scratch.scopeCandidate = true;
 
@@ -867,7 +869,7 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 				{
 					auto textType = INTL_texttype_lookup(tdbb, INTL_INDEX_TO_TEXT(iType));
 
-					if (segment.scanType != segmentScanMissing && !(idx->idx_flags & idx_unique))
+					if (scanType != segmentScanMissing && !(idx->idx_flags & idx_unique))
 					{
 						if (textType->getFlags() & TEXTTYPE_SEPARATE_UNIQUE)
 						{
@@ -878,7 +880,7 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 						}
 					}
 
-					if (segment.scanType == segmentScanStarting)
+					if (scanType == segmentScanStarting)
 					{
 						if (textType->getFlags() & TEXTTYPE_MULTI_STARTING_KEY)
 							scratch.useMultiStartingKeys = true;	// use INTL_KEY_MULTI_STARTING
@@ -887,51 +889,63 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 					}
 				}
 
-				if (const auto list = segment.valueList)
-				{
-					fb_assert(segment.scanType == segmentScanList);
+				auto selectivity = idx->idx_rpt[j].idx_selectivity;
+				const auto useDefaultSelectivity = (selectivity <= 0);
 
+				// When the index selectivity is zero then the statement is prepared
+				// on an empty table or the statistics aren't updated. So assume every
+				// match to represent 1/10 of the maximum selectivity.
+				if (useDefaultSelectivity)
+					selectivity = MAX(scratch.selectivity * DEFAULT_SELECTIVITY, minSelectivity);
+
+				if (scanType == segmentScanList)
+				{
 					if (listCount) // we cannot have more than one list matched to an index
 						break;
 
+					const auto list = segment.valueList;
+					fb_assert(list);
+
 					listCount = list->getCount();
 					maxSelectivity = scratch.selectivity;
 				}
 
 				// Check if this is the last usable segment
 				if (!scratch.usePartialKey &&
-					(segment.scanType == segmentScanEqual ||
-					 segment.scanType == segmentScanEquivalent ||
-					 segment.scanType == segmentScanMissing ||
-					 segment.scanType == segmentScanList))
+					(scanType == segmentScanEqual ||
+					 scanType == segmentScanEquivalent ||
+					 scanType == segmentScanMissing ||
+					 scanType == segmentScanList))
 				{
 					// This is a perfect usable segment thus update root selectivity
 					scratch.lowerCount++;
 					scratch.upperCount++;
-					scratch.selectivity = idx->idx_rpt[j].idx_selectivity;
 					scratch.nonFullMatchedSegments = idx->idx_count - (j + 1);
 					// Add matches for this segment to the main matches list
 					matches.join(segment.matches);
+					scratch.selectivity = selectivity;
 
 					// An equality scan for any unique index cannot retrieve more
 					// than one row. The same is true for an equivalence scan for
-					// any primary index.
-					const bool single_match =
-						(segment.scanType == segmentScanEqual &&
-							(idx->idx_flags & idx_unique)) ||
-						(segment.scanType == segmentScanEquivalent &&
-							(idx->idx_flags & idx_primary));
+					// any primary index. A missing scan for any primary index is
+					// known to return no rows, but let's treat it the same way.
+					const bool uniqueMatch =
+						(scanType == segmentScanEqual && (idx->idx_flags & idx_unique)) ||
+						(scanType == segmentScanEquivalent && (idx->idx_flags & idx_primary)) ||
+						(scanType == segmentScanMissing && (idx->idx_flags & idx_primary));
 
-					// dimitr: IS NULL scan against primary key is guaranteed
-					//		   to return zero rows. Do we need yet another
-					//		   special case here?
-
-					if (single_match && ((j + 1) == idx->idx_count))
+					if (uniqueMatch && ((j + 1) == idx->idx_count))
 					{
 						// We have found a full equal matching index and it's unique,
 						// so we can stop looking further, because this is the best
 						// one we can get.
 						unique = true;
+
+						// If selectivity is assumed, a better guess for the unique match
+						// would be 1 / cardinality
+						if (useDefaultSelectivity)
+							scratch.selectivity = minSelectivity;
+
 						break;
 					}
 
@@ -943,58 +957,54 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 				}
 				else
 				{
-					// This is our last segment that we can use,
-					// estimate the selectivity
-					auto selectivity = scratch.selectivity;
-					double factor = 1;
-
-					switch (segment.scanType)
+					if (scanType != segmentScanNone)
 					{
-						case segmentScanBetween:
-							scratch.lowerCount++;
-							scratch.upperCount++;
-							selectivity = idx->idx_rpt[j].idx_selectivity;
-							factor = REDUCE_SELECTIVITY_FACTOR_BETWEEN;
-							break;
+						// This is our last segment that we can use,
+						// estimate the selectivity
+						double factor = 1;
 
-						case segmentScanLess:
-							scratch.upperCount++;
-							selectivity = idx->idx_rpt[j].idx_selectivity;
-							factor = REDUCE_SELECTIVITY_FACTOR_LESS;
-							break;
+						switch (scanType)
+						{
+							case segmentScanBetween:
+								scratch.lowerCount++;
+								scratch.upperCount++;
+								factor = REDUCE_SELECTIVITY_FACTOR_BETWEEN;
+								break;
 
-						case segmentScanGreater:
-							scratch.lowerCount++;
-							selectivity = idx->idx_rpt[j].idx_selectivity;
-							factor = REDUCE_SELECTIVITY_FACTOR_GREATER;
-							break;
+							case segmentScanLess:
+								scratch.upperCount++;
+								factor = REDUCE_SELECTIVITY_FACTOR_LESS;
+								break;
 
-						case segmentScanStarting:
-						case segmentScanEqual:
-						case segmentScanEquivalent:
-							scratch.lowerCount++;
-							scratch.upperCount++;
-							selectivity = idx->idx_rpt[j].idx_selectivity;
-							factor = REDUCE_SELECTIVITY_FACTOR_STARTING;
-							break;
+							case segmentScanGreater:
+								scratch.lowerCount++;
+								factor = REDUCE_SELECTIVITY_FACTOR_GREATER;
+								break;
 
-						default:
-							fb_assert(segment.scanType == segmentScanNone);
-							break;
-					}
+							case segmentScanStarting:
+							case segmentScanEqual:
+							case segmentScanEquivalent:
+							case segmentScanList:
+								scratch.lowerCount++;
+								scratch.upperCount++;
+								factor = REDUCE_SELECTIVITY_FACTOR_STARTING;
+								break;
 
-					// Adjust the compound selectivity using the reduce factor.
-					// It should be better than the previous segment but worse
-					// than a full match.
-					const double diffSelectivity = scratch.selectivity - selectivity;
-					selectivity += (diffSelectivity * factor);
-					fb_assert(selectivity <= scratch.selectivity);
-					scratch.selectivity = selectivity;
+							default:
+								fb_assert(false);
+								break;
+						}
+
+						// Adjust the compound selectivity using the reduce factor.
+						// It should be better than the previous segment but worse
+						// than a full match.
+						const double diffSelectivity = scratch.selectivity - selectivity;
+						selectivity += (diffSelectivity * factor);
+						fb_assert(selectivity <= scratch.selectivity);
+						scratch.selectivity = selectivity;
 
-					if (segment.scanType != segmentScanNone)
-					{
-						matches.join(segment.matches);
 						scratch.nonFullMatchedSegments = idx->idx_count - j;
+						matches.join(segment.matches);
 					}
 
 					break;
@@ -1004,19 +1014,7 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 			if (scratch.scopeCandidate)
 			{
 				double selectivity = scratch.selectivity;
-
-				// When the index selectivity is zero then the statement is prepared
-				// on an empty table or the statistics aren't updated. The priorly
-				// calculated selectivity is meaningless in this case. So instead:
-				// - for an unique match, estimate the selectivity via the stream cardinality;
-				// - for a non-unique one, assume 1/10 of the maximum selectivity, so that
-				//   at least some indexes could be utilized by the optimizer.
-				if (idx->idx_selectivity <= 0)
-				{
-					selectivity = unique ?
-						MIN(MAXIMUM_SELECTIVITY / cardinality, DEFAULT_SELECTIVITY) :
-						DEFAULT_SELECTIVITY;
-				}
+				fb_assert(selectivity);
 
 				// Calculate the cost (only index pages) for this index
 				auto cost = DEFAULT_INDEX_COST + selectivity * scratch.cardinality;
@@ -1067,6 +1065,7 @@ void Retrieval::getInversionCandidates(InversionCandidateList& inversions,
 			invCandidate->cost = DEFAULT_INDEX_COST + scratch.cardinality;
 			invCandidate->indexes = 1;
 			invCandidate->scratch = &scratch;
+			invCandidate->nonFullMatchedSegments = scratch.segments.getCount();
 			invCandidate->matches.assign(scratch.matches);
 
 			for (auto match : invCandidate->matches)
@@ -1190,15 +1189,31 @@ InversionNode* Retrieval::makeIndexScanNode(IndexScratch* indexScratch) const
 					fb_assert(!retrieval->irb_list);
 					retrieval->irb_list = segments[i].valueList;
 				}
+
+				if (segments[i].scale)
+				{
+					if (!retrieval->irb_scale)
+					{
+						retrieval->irb_scale = FB_NEW_POOL(getPool()) SSHORT[count];
+						memset(retrieval->irb_scale, 0, sizeof(SSHORT) * count);
+					}
+					retrieval->irb_scale[i] = segments[i].scale;
+				}
 			}
 		}
 
 		// This index is never used for IS NULL, thus we can ignore NULLs
 		// already at index scan. But this rule doesn't apply to nod_equiv
 		// which requires NULLs to be found in the index.
-		// A second exception is when this index is used for navigation.
-		if (ignoreNullsOnScan && !(idx->idx_runtime_flags & idx_navigate))
+		//
+		// dimitr:	make sure the check below is never moved outside the IF scope,
+		// 			as this flag must not be set for a full index scan,
+		//			see also the assertion below
+		if (ignoreNullsOnScan)
+		{
+			fb_assert(indexScratch->lowerCount || indexScratch->upperCount);
 			retrieval->irb_generic |= irb_ignore_null_value_key;
+		}
 
 		const auto& lastSegment = segments[count - 1];
 
@@ -1230,6 +1245,9 @@ InversionNode* Retrieval::makeIndexScanNode(IndexScratch* indexScratch) const
 	// Check to see if this is really an equality retrieval
 	if (retrieval->irb_lower_count == retrieval->irb_upper_count)
 	{
+		const bool fullMatch = (retrieval->irb_lower_count == idx->idx_count);
+		bool uniqueMatch = false;
+
 		retrieval->irb_generic |= irb_equality;
 
 		for (unsigned i = 0; i < retrieval->irb_lower_count; i++)
@@ -1239,7 +1257,22 @@ InversionNode* Retrieval::makeIndexScanNode(IndexScratch* indexScratch) const
 				retrieval->irb_generic &= ~irb_equality;
 				break;
 			}
+
+			if (segments[i].scanType == segmentScanMissing ||
+				segments[i].scanType == segmentScanEquivalent)
+			{
+				if (fullMatch && (idx->idx_flags & idx_primary))
+					uniqueMatch = true;
+			}
+			else if (segments[i].scanType == segmentScanEqual)
+			{
+				if (fullMatch && (idx->idx_flags & idx_unique))
+					uniqueMatch = true;
+			}
 		}
+
+		if ((retrieval->irb_generic & irb_equality) && uniqueMatch)
+			retrieval->irb_generic |= irb_unique;
 	}
 
 	// If we are matching less than the full index, this is a partial match
@@ -1360,66 +1393,90 @@ InversionCandidate* Retrieval::makeInversion(InversionCandidateList& inversions)
 					invCandidate->nonFullMatchedSegments = 0;
 					invCandidate->matchedSegments = currentInv->matchedSegments;
 					invCandidate->dependencies = currentInv->dependencies;
-					matches.clear();
 
 					for (const auto currentMatch : currentInv->matches)
 					{
-						if (!matches.exist(currentMatch))
-							matches.add(currentMatch);
+						if (!invCandidate->matches.exist(currentMatch))
+							invCandidate->matches.add(currentMatch);
 					}
 
-					if (currentInv->boolean)
+					if (const auto currentMatch = currentInv->boolean)
 					{
-						if (!matches.exist(currentInv->boolean))
-							matches.add(currentInv->boolean);
+						if (!invCandidate->matches.exist(currentMatch))
+							invCandidate->matches.add(currentMatch);
 					}
 
-					invCandidate->matches.join(matches);
+					matches.assign(invCandidate->matches);
+
 					if (customPlan)
 						continue;
 
 					return invCandidate;
 				}
 
-				// Look if a match is already used by previous matches.
-				bool anyMatchAlreadyUsed = false, matchUsedByNavigation = false;
-				for (const auto currentMatch : currentInv->matches)
+				if (!customPlan)
 				{
-					if (matches.exist(currentMatch))
-					{
-						anyMatchAlreadyUsed = true;
+					// Look if a match is already used by previous matches
+					bool anyMatchAlreadyUsed = false, matchUsedByNavigation = false;
 
-						if (navigationCandidate &&
-							navigationCandidate->matches.exist(currentMatch))
-						{
-							matchUsedByNavigation = true;
-						}
-
-						break;
-					}
-				}
-
-				if (currentInv->boolean && matches.exist(currentInv->boolean))
-					anyMatchAlreadyUsed = true;
-
-				if (anyMatchAlreadyUsed && !customPlan)
-				{
-					currentInv->used = true;
-
-					if (matchUsedByNavigation)
-						continue;
-
-					// If a match on this index was already used by another
-					// index, add also the other matches from this index.
 					for (const auto currentMatch : currentInv->matches)
 					{
-						if (!matches.exist(currentMatch))
-							matches.add(currentMatch);
+						if (matches.exist(currentMatch))
+						{
+							anyMatchAlreadyUsed = true;
+
+							if (navigationCandidate &&
+								navigationCandidate->matches.exist(currentMatch))
+							{
+								matchUsedByNavigation = true;
+							}
+
+							break;
+						}
 					}
 
-					// Restart loop, because other indexes could also be excluded now.
-					restartLoop = true;
-					break;
+					if (const auto currentMatch = currentInv->boolean)
+					{
+						if (matches.exist(currentMatch))
+						{
+							anyMatchAlreadyUsed = true;
+
+							if (navigationCandidate &&
+								navigationCandidate->matches.exist(currentMatch))
+							{
+								matchUsedByNavigation = true;
+							}
+						}
+						else if (matchUsedByNavigation)
+							anyMatchAlreadyUsed = false;
+					}
+
+					// If some match was already used by another index, skip this index
+
+					if (anyMatchAlreadyUsed)
+					{
+						if (!matchUsedByNavigation)
+						{
+							// Add the other matches from this index
+
+							for (const auto currentMatch : currentInv->matches)
+							{
+								if (!matches.exist(currentMatch))
+									matches.add(currentMatch);
+							}
+
+							if (const auto currentMatch = currentInv->boolean)
+							{
+								if (!matches.exist(currentMatch))
+									matches.add(currentMatch);
+							}
+						}
+
+						// Restart loop, because other indexes could also be excluded now
+						currentInv->used = true;
+						restartLoop = true;
+						break;
+					}
 				}
 
 				if (!bestCandidate)
@@ -1532,16 +1589,19 @@ InversionCandidate* Retrieval::makeInversion(InversionCandidateList& inversions)
 					invCandidate->dependencies = bestCandidate->dependencies;
 					invCandidate->condition = bestCandidate->condition;
 
-					for (FB_SIZE_T j = 0; j < bestCandidate->matches.getCount(); j++)
+					for (const auto bestMatch : bestCandidate->matches)
 					{
-						if (!matches.exist(bestCandidate->matches[j]))
-							matches.add(bestCandidate->matches[j]);
+						if (!invCandidate->matches.exist(bestMatch))
+							invCandidate->matches.add(bestMatch);
 					}
-					if (bestCandidate->boolean)
+
+					if (const auto bestMatch = bestCandidate->boolean)
 					{
-						if (!matches.exist(bestCandidate->boolean))
-							matches.add(bestCandidate->boolean);
+						if (!invCandidate->matches.exist(bestMatch))
+							invCandidate->matches.add(bestMatch);
 					}
+
+					matches.join(invCandidate->matches);
 				}
 				else if (!bestCandidate->condition)
 				{
@@ -1568,15 +1628,17 @@ InversionCandidate* Retrieval::makeInversion(InversionCandidateList& inversions)
 
 					for (const auto bestMatch : bestCandidate->matches)
 					{
-						if (!matches.exist(bestMatch))
-	                        matches.add(bestMatch);
+						if (!invCandidate->matches.exist(bestMatch))
+							invCandidate->matches.add(bestMatch);
 					}
 
-					if (bestCandidate->boolean)
+					if (const auto bestMatch = bestCandidate->boolean)
 					{
-						if (!matches.exist(bestCandidate->boolean))
-							matches.add(bestCandidate->boolean);
+						if (!invCandidate->matches.exist(bestMatch))
+							invCandidate->matches.add(bestMatch);
 					}
+
+					matches.join(invCandidate->matches);
 				}
 
 				if (invCandidate->unique)
@@ -1610,10 +1672,13 @@ InversionCandidate* Retrieval::makeInversion(InversionCandidateList& inversions)
 		invCandidate->cost += navigationCandidate->cost;
 		++invCandidate->indexes;
 		invCandidate->navigated = true;
-	}
 
-	if (invCandidate)
-		invCandidate->matches.join(matches);
+		for (const auto navMatch : navigationCandidate->matches)
+		{
+			if (!invCandidate->matches.exist(navMatch))
+				invCandidate->matches.add(navMatch);
+		}
+	}
 
 	return invCandidate;
 }
@@ -1748,9 +1813,6 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 
 	const bool isDesc = (idx->idx_flags & idx_descending);
 
-	// Needed for int64 matches, see injectCast() function
-	CastNode *cast = nullptr, *cast2 = nullptr;
-
 	fb_assert(indexScratch->segments.getCount() == idx->idx_count);
 
 	for (unsigned i = 0; i < idx->idx_count; i++)
@@ -1777,8 +1839,8 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 						(segment->scanType == segmentScanEquivalent) ||
 						(segment->scanType == segmentScanList)))
 					{
-						segment->lowerValue = injectCast(csb, value, cast, matchDesc);
-						segment->upperValue = injectCast(csb, value2, cast2, matchDesc);
+						segment->lowerValue = value;
+						segment->upperValue = value2;
 						segment->scanType = segmentScanBetween;
 						segment->excludeLower = false;
 						segment->excludeUpper = false;
@@ -1791,8 +1853,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 					// override it with worser matches.
 					if (!(segment->scanType == segmentScanEqual))
 					{
-						segment->lowerValue = segment->upperValue =
-							injectCast(csb, value, cast, matchDesc);
+						segment->lowerValue = segment->upperValue = value;
 						segment->scanType = segmentScanEquivalent;
 						segment->excludeLower = false;
 						segment->excludeUpper = false;
@@ -1801,8 +1862,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 
 				case blr_eql:
 					segment->matches.add(boolean);
-					segment->lowerValue = segment->upperValue =
-						injectCast(csb, value, cast, matchDesc);
+					segment->lowerValue = segment->upperValue = value;
 					segment->scanType = segmentScanEqual;
 					segment->excludeLower = false;
 					segment->excludeUpper = false;
@@ -1839,7 +1899,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 
 						if (forward)
 						{
-							segment->lowerValue = injectCast(csb, value, cast, matchDesc);
+							segment->lowerValue = value;
 							if (segment->scanType == segmentScanLess)
 								segment->scanType = segmentScanBetween;
 							else
@@ -1847,7 +1907,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 						}
 						else
 						{
-							segment->upperValue = injectCast(csb, value, cast, matchDesc);
+							segment->upperValue = value;
 							if (segment->scanType == segmentScanGreater)
 								segment->scanType = segmentScanBetween;
 							else
@@ -1871,7 +1931,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 
 						if (forward)
 						{
-							segment->upperValue = injectCast(csb, value, cast, matchDesc);
+							segment->upperValue = value;
 							if (segment->scanType == segmentScanGreater)
 								segment->scanType = segmentScanBetween;
 							else
@@ -1879,7 +1939,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 						}
 						else
 						{
-							segment->lowerValue = injectCast(csb, value, cast, matchDesc);
+							segment->lowerValue = value;
 							if (segment->scanType == segmentScanLess)
 								segment->scanType = segmentScanBetween;
 							else
@@ -1896,8 +1956,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 					if (!((segment->scanType == segmentScanEqual) ||
 						(segment->scanType == segmentScanEquivalent)))
 					{
-						segment->lowerValue = segment->upperValue =
-							injectCast(csb, value, cast, matchDesc);
+						segment->lowerValue = segment->upperValue = value;
 						segment->scanType = segmentScanStarting;
 						segment->excludeLower = false;
 						segment->excludeUpper = false;
@@ -1915,11 +1974,7 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 				(segment->scanType == segmentScanEquivalent)))
 			{
 				fb_assert(listNode->lookup);
-				for (auto& item : *listNode->lookup)
-				{
-					cast = nullptr; // create new cast node for every value
-					item = injectCast(csb, item, cast, matchDesc);
-				}
+
 				segment->lowerValue = segment->upperValue = nullptr;
 				segment->valueList = listNode->lookup;
 				segment->scanType = segmentScanList;
@@ -1945,6 +2000,18 @@ bool Retrieval::matchBoolean(IndexScratch* indexScratch,
 			return false;
 		}
 
+		// Scale for big exact numerics
+		if (!missingNode)
+		{
+			switch (matchDesc.dsc_dtype)
+			{
+			case dtype_int64:
+			case dtype_int128:
+				segment->scale = matchDesc.dsc_scale;
+				break;
+			}
+		}
+
 		// A match could be made
 		if (segment->scope < scope)
 			segment->scope = scope;
@@ -2128,7 +2195,7 @@ InversionCandidate* Retrieval::matchDbKey(BoolExprNode* boolean) const
 			{
 				const auto inversion = FB_NEW_POOL(getPool()) InversionNode(value, n);
 				inversion->impure = csb->allocImpure<impure_inversion>();
-				listInversion = composeInversion(listInversion, inversion, InversionNode::TYPE_IN);
+				listInversion = composeInversion(listInversion, inversion, InversionNode::TYPE_OR);
 			}
 
 			invCandidate->inversion = listInversion;
@@ -2252,7 +2319,10 @@ InversionCandidate* Retrieval::matchOnIndexes(IndexScratchList& inputIndexScratc
 					matches.add(match);
 
 				for (const auto match : invCandidate2->matches)
-					matches.add(match);
+				{
+					if (matches.exist(match))
+						invCandidate->matches.add(match);
+				}
 			}
 
 			return invCandidate;
