@@ -81,9 +81,7 @@ namespace
 
 	void configError(const string& type, const string& key, const string& value)
 	{
-		string msg;
-		msg.printf("%s specifies %s: %s", key.c_str(), type.c_str(), value.c_str());
-		raiseError(msg.c_str());
+		raiseError("%s specifies %s: %s", key.c_str(), type.c_str(), value.c_str());
 	}
 
 	void checkAccess(const PathName& path, const string& key)
