@@ -94,10 +94,10 @@ TracePluginImpl::TracePluginImpl(IPluginBase* plugin,
 	logWriter(initInfo->getLogWriter()),
 	config(configuration),
 	record(*getDefaultMemoryPool()),
-	connections(getDefaultMemoryPool()),
-	transactions(getDefaultMemoryPool()),
-	statements(getDefaultMemoryPool()),
-	services(getDefaultMemoryPool()),
+	connections(*getDefaultMemoryPool()),
+	transactions(*getDefaultMemoryPool()),
+	statements(*getDefaultMemoryPool()),
+	services(*getDefaultMemoryPool()),
 	routines(*getDefaultMemoryPool()),
 	include_codes(*getDefaultMemoryPool()),
 	exclude_codes(*getDefaultMemoryPool())
